import random

from django.contrib.auth.admin import UserAdmin
from django.utils.decorators import method_decorator
from django.views.generic import TemplateView
from django.contrib.auth.decorators import login_required


from .counters import CounterBase
from .charts import ChartsBase


class ValiDashboardBase(TemplateView):
  extra_context_data = []
  list_counters = []
  list_charts = []

  def get_list_counters(self):
    list_counters = []
    for counter in self.list_counters:
      if isinstance(counter, CounterBase):
        list_counters.append(
          {
            'title': counter.get_title(self.request),
            'value': counter.get_value(self.request),
            'style': counter.get_style(self.request),
            'icon': counter.get_icon(self.request),
          }
        )
      else:
        list_counters.append(counter)
    return list_counters

  def get_list_charts(self):
    list_charts = []
    for chart in self.list_charts:
      if isinstance(chart, ChartsBase):
        list_charts.append(
          {
            'title': chart.get_title(self.request),
            'name': chart.get_name(self.request),
            'chart_type': chart.get_chart_type(self.request),
            'labels': chart.get_labels(self.request),
            'datasets': chart.get_datasets(self.request),
          }
        )
      else:
        list_charts.append(chart)
    return list_charts

  def get_extra_context_data(self):
    return self.extra_context

  def get_context_data(self, **kwargs):
    context = super().get_context_data(**kwargs)
    context['counters'] = self.get_list_counters()
    context['charts'] = self.get_list_charts()
    return context

  def get_template_names(self):
    return super().get_template_names()

  @method_decorator(login_required)
  def dispatch(self, request, *args, **kwargs):
    return super(ValiDashboardBase, self).dispatch(request, *args, **kwargs)

"""
def usuariosAtivos():
  count = 0

  cadastros = Cadastros.objects.all()

  for cadastro in cadastros:
    try:
      Turmas.objects.filter(alunos=cadastro)
      count = count + 1

    except Turmas.DoesNotExist:
      pass

  return count


def usuariosInativos():
  cadastrosTotais = Cadastros.objects.count()
  count = cadastrosTotais - usuariosAtivos()
  return count


def pagamentosAberto():
  result = Pagamentos.objects.filter(pago=False)
  return result.count()


def vendas():
  quantidadeVendas = Venda.objects.count()
  return quantidadeVendas

"""
class ValiDashboardView(ValiDashboardBase):

  r = lambda: random.randint(0, 100)

  template_name = 'dashboard.html'
  # default count users data
  """
  users = usuariosAtivos()
  # default count groups data
  userInat = usuariosInativos()
  # default count apps data
  pagamentosEmAberto = pagamentosAberto()

  vendas = vendas()
  """
  # default charts data
  list_charts = [

    {
      # Support Chart types: PolarArea, Pie, Doughnut
      "name": "piechart1",
      "title": "Faltas",
      "chart_type": "Pie",
      "datasets": [

        {
          "value": 300,
          "color": '#%02X%02X%02X' % (r(), r(), r()),
          "label": "Red"
        },
        {
          "value": 50,
          "color": '#%02X%02X%02X' % (r(), r(), r()),
          "label": "Green"
        },
        {
          "value": 100,
          "color": '#%02X%02X%02X' % (r(), r(), r()),
          "label": "Yellow"
        },
      ]
    },
    {
      # Support Chart types: Bar, Line, Radar
      "name": "vendas",
      "title": "Vendas Mensais",
      "chart_type": "Bar",
      "labels": ["2018-03-01", "2018-03-02", "2018-03-03", "2018-03-04", "2018-03-05"],
      "datasets": [
        {
          "label": "dataset 1",
          "fillColor": "rgba(220,220,220,0.2)",
          "strokeColor": "rgba(220,220,220,1)",
          "pointColor": "rgba(220,220,220,1)",
          "pointStrokeColor": "#fff",
          "pointHighlightFill": "#fff",
          "pointHighlightStroke": "rgba(220,220,220,1)",
          "data": [65, 59, 80, 81, 80]
        },
        {
          "label": "dataset 2",
          "fillColor": "rgba(151,187,205,0.2)",
          "strokeColor": "rgba(151,187,205,1)",
          "pointColor": "rgba(151,187,205,1)",
          "pointStrokeColor": "#fff",
          "pointHighlightFill": "#fff",
          "pointHighlightStroke": "rgba(151,187,205,1)",
          "data": [28, 48, 40, 19, 69]
        }
      ],
    },

  ]

  # default icons data
  list_counters = [
    {"title": "ATIVOS", "value": 4, "style": "primary", "icon": "fa-user-circle"},
    {"title": "INATIVOS", "value": 6, "style": "warning", "icon": "fa-users"},
    {"title": "PENDENTES", "value": 8, "style": "info", "icon": "fa-money"},
    {"title": "VENDAS", "value": 10, "style": "danger", "icon": "fa-line-chart"},
  ]


class ValiChangePassword(UserAdmin):
  template_name = "admin/auth/user/change_password.html"





class CounterBase(object):
    title = None
    value = None
    style = None
    icon = None

    def get_title(self, request):
        return self.title
    
    def get_value(self, request):
        return self.value
    
    def get_style(self, request):
        return self.style if self.style else 'primary'
    
    def get_icon(self, request):
        return self.icon if self.icon else 'fa-tasks'


class ModelCounter(CounterBase):
    model = None

    def __init__(self):
        if not self.title:
            self.title = self.model._meta.verbose_name_plural

    def get_value(self, request):
        return self.model.objects.count()



from django.contrib.admin.widgets import RelatedFieldWidgetWrapper, FilteredSelectMultiple
from django.forms.widgets import CheckboxSelectMultiple
from django import forms


class ValiDashboardWidget(forms.Widget):
    def render(self, name, value, attrs=None, renderer=None):
        #TODO
        return 'render'


class ValiDateWidget(forms.Widget):
    def render(self, name, value, attrs=None, renderer=None):
        #TODO
        return 'dateinput'


class ValiDateTimeWidget(forms.Widget):
    def render(self, name, value, attrs=None, renderer=None):
        #TODO
        return 'datetimeinput'


# fork FilteredSelectMultiple
class ValiFilteredSelectMultiple(FilteredSelectMultiple):
    """ customize FilteredSelectMultiple, not used for now """

    def get_context(self, name, value, attrs):
        context = super(ValiFilteredSelectMultiple, self).get_context(name, value, attrs)
        context['widget']['attrs']['class'] = 'selectfilter'
        if self.is_stacked:
            context['widget']['attrs']['class'] += 'stacked'
        context['widget']['attrs']['data-field-name'] = self.verbose_name
        context['widget']['attrs']['data-is-stacked'] = int(self.is_stacked)
        return context


class ValiCheckboxSelectMultiple(CheckboxSelectMultiple):
    def get_context(self, name, value, attrs):
        context = super(ValiCheckboxSelectMultiple, self).get_context(name, value, attrs)
        context['widget']['attrs']['class'] = 'vali-multicheckbox list-group'
        return context


class ValiRelatedFieldWidgetWrapper(RelatedFieldWidgetWrapper):
    """ customize ValiRelatedFieldWidgetWrapper, display auth.user.permissions in multiple checkbox """

    def __init__(self, *args, **kwargs):
        super(ValiRelatedFieldWidgetWrapper, self).__init__(*args, **kwargs)
        self.widget = ValiCheckboxSelectMultiple(attrs=self.widget.attrs, choices=self.widget.choices)



from django.contrib import admin


def vali_models_group(groupname=''):
    """
    assign menu_group name to ModalAdmin instance,
    admin index display registered models group by menu_group
    :param groupname:
    :return:
    """

    def _models_group(cls):
        if issubclass(cls, admin.ModelAdmin) and groupname:
            cls.model_group = groupname
        return cls

    return _models_group




class ChartsBase(object):
    chart_type = None
    name = None
    title = None
    labels = None
    datasets = None

    def get_chart_type(self, request):
        return self.chart_type

    def get_name(self, request):
        return self.name

    def get_title(self, request):
        return self.title

    def get_labels(self, request):
        return self.labels

    def get_datasets(self, request):
        return self.datasets


class ModelCharts(ChartsBase):
    model = None

    def __init__(self):
        if not self.title:
            self.title = self.model._meta.verbose_name + 'Data'

        if not self.chart_type:
            self.chart_type = 'Bar'

        if not self.name:
            self.name = self.chart_type + 'chart'
